﻿namespace SdkCoreDemo
{
  using System;
  using System.Collections.Generic;
  using System.Linq;
  using System.Net;
  using System.Threading;

  using Knx.Bus.Common.Configuration;
  using Knx.Bus.Common.KnxIp;

  using Knx.Falcon.Sdk;

#if LOG
  using log4net;
  using log4net.Config;
#endif

  public class Program
  {
    static void Main(string[] parameter)
    {
#if LOG
      if (parameter.Contains("-log"))
      {
        BasicConfigurator.Configure(LogManager.GetRepository(typeof(Program).Assembly));
      }
#endif

      if (parameter.Contains("-slow"))
      {
        ////For debugging and attaching purposes
        Thread.Sleep(5000);
      }

      var discoveryResults = DiscoverIp();
      
      List<UsbConnectorParameters> usbDeviceList = new List<UsbConnectorParameters>();
      try
      {
        usbDeviceList.AddRange(UsbDeviceEnumerator.GetAvailableInterfaces(TimeSpan.FromSeconds(2)));
        if (usbDeviceList.Any())
        {
          Console.WriteLine();
          Console.WriteLine("USB Devices:");
        }
        foreach (var dev in usbDeviceList)
        {
          Console.WriteLine((discoveryResults.Count + usbDeviceList.IndexOf(dev)) + ": " + dev.Name + " (" + dev.HidPath + ")" + " AddressToOpen:" + dev.PreferredAddressToOpen);
        }
      }
      catch (Exception ex)
      {
        Console.WriteLine("Could not enumerate USB devices ->" + ex.Message);
      }
      
      Console.WriteLine();
      Console.WriteLine("Select Interface or enter Ip Address:");
      List<ConnectorParameters> connectorParameterses = new List<ConnectorParameters>();
      connectorParameterses.AddRange(discoveryResults.Select(me => new KnxIpTunnelingConnectorParameters(me.IpAddress.ToString(), 0x0e57, false)));
      connectorParameterses.AddRange(usbDeviceList);
      ConnectorParameters selectedConnector = null;
      while (selectedConnector == null)
      {
        var input = Console.ReadLine();
        if (int.TryParse(input, out int parsed))
        {
          selectedConnector = connectorParameterses[parsed];
        }
        else if (IPAddress.TryParse(input, out var address))
        {
          selectedConnector = new KnxIpTunnelingConnectorParameters(address.ToString(), 0x0e57, false);
        }
      }

      using (var bus = new Bus(selectedConnector))
      {
        bus.Connect();
        if (bus.IsConnected)
        {
          Console.WriteLine("Connected to " + bus.OpenParameters);
        }

        var senderAddress = bus.LocalIndividualAddress;
        Console.WriteLine("LocalIndividualAddress: " + senderAddress);
        bus.GroupValueReceived += args =>
          {
            Console.WriteLine("IndividualAddress: " + args.IndividualAddress + " Value: " + args.Value + " Address:" + args.Address);
          };

        while (true) { }
      }
    }

    private static List<DiscoveryResult> DiscoverIp()
    {
      var client = new DiscoveryClient(AdapterTypes.All);
      try
      {
        var networks = new NetworkAdapterEnumerator(AdapterTypes.All).ToList();
        Console.WriteLine("Begin Discover on:");
        foreach (var network in networks)
        {
          Console.WriteLine(network.Name + " " + network.IpV4Addresses.FirstOrDefault());
        }
      }
      catch (Exception e)
      {
        Console.WriteLine("Could not iterate NetworkAdapter! Error: " + e.Message);
      }

      Console.WriteLine();
      Console.WriteLine("Discover Results:");
      try
      {

        var discoveryResults = client.Discover().ToList();
        foreach (var result in discoveryResults)
        {
          Console.WriteLine((discoveryResults.IndexOf(result)) + ": " + result.FriendlyName + " (" + result.IpAddress + ")");
        }
        return discoveryResults;
      }
      catch (Exception e)
      {
        Console.WriteLine("Could not discover! Error: " + e.Message);
      }

      return new List<DiscoveryResult>();
    }
  }
}
